@@ -1,6 +1,6 @@
 import logging
 from enum import Enum, auto
-from typing import Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 
 from pydantic import BaseModel, SecretStr, validator
 
@@ -21,6 +21,7 @@ class ChartValuesFormat(Enum):
     def __str__(self):
         return self.name
 
+
 class ResourceChartItemType(Enum):
     """
     Item selection for Alert resource enricher
@@ -70,6 +71,197 @@ class ActionParams(DocumentedModel):
     pass
 
 
+class ResourceInfo(BaseModel):
+    name: Optional[str] = "unresolved"
+    namespace: Optional[str]
+    kind: Optional[str]
+    node: Optional[str]
+    container: Optional[str]
+    cluster: Optional[str]
+
+
+class HolmesParams(ActionParams):
+    holmes_url: Optional[str]
+    model: Optional[str]
+    @validator("holmes_url", allow_reuse=True)
+    def validate_protocol(cls, v):
+        if v and not v.startswith("http"):  # if the user configured url without http(s)
+            v = f"http://{v}"
+            logging.info(f"Adding protocol to holmes_url: {v}")
+        return v
+
+
+class AIInvestigateParams(HolmesParams):
+    """
+    :var resource: The resource related to this investigation. A resource has a `name` and `kind`, and may have `namespace` and `node`
+    :var investigation_type: The type of investigation: issue/analyze_problems
+    :var runbooks: List of human readable recommended runbooks that holmes can use for the investigation.
+    :var ask: Override question to ask holmes
+    :var context: Additional information that can assist with the investigation
+    :var sections: Sections that Holmes should return. Dictionary where the key is the section's title and the value a description for Holmes (LLM)
+
+    :example ask: What are all the issues in my cluster right now?
+    :example runbooks: ["Try to get the pod logs and find errors", "get the pod yaml and check if there are finalizers"]
+    """
+
+    resource: Optional[ResourceInfo]
+    investigation_type: str
+    runbooks: Optional[List[str]]
+    ask: Optional[str]
+    context: Optional[Dict[str, Any]]
+    sections: Optional[Dict[str, str]] = None
+    stream: bool = False
+
+
+class HolmesToolsResult(BaseModel):
+    """
+    :var name: Name of the tool.
+    :var description: Description of the tool.
+    :var output: Output of the tool.
+    """
+
+    name: str
+    description: str
+    output: str
+
+
+class HolmesInvestigationResult(BaseModel):
+    """
+    :var result: A dictionary containing the summary of the issue investigation.
+    :var tools: A list of dictionaries where each dictionary contains information
+                about the tool, its name, description and output.
+
+    It is based on the holmes investigation saved to Evidence table.
+    """
+
+    result: str
+    tools: Optional[List[HolmesToolsResult]] = []
+
+
+class HolmesConversationHistory(BaseModel):
+    """
+    :var ask: A prompt sent by user.
+    :var answer: HolmesInvestigationResult object that contains result of holmes_conversation action investigation
+                 for the prompt.
+    """
+
+    ask: str
+    answer: HolmesInvestigationResult
+
+
+class HolmesIssueChatParamsContext(BaseModel):
+    """
+    :var investigation_result: HolmesInvestigationResult object that contains investigation saved to Evidence table by frontend for the issue.
+    :var issue_type: aggregation key of the issue
+    :var robusta_issue_id: id of the issue
+    :var labels: labels from the issue
+    """
+
+    investigation_result: HolmesInvestigationResult
+    issue_type: str
+    robusta_issue_id: Optional[str] = None
+    labels: Optional[Dict[str, str]] = None
+
+
+# will be deprecated later alongside with holmes_conversation action
+class HolmesOldConversationIssueContext(HolmesIssueChatParamsContext):
+    """
+    :var conversation_history: List of HolmesConversationHistory objects that contain previous user prompts and responses.
+    :var source: source of the issue
+    """
+
+    conversation_history: Optional[list[HolmesConversationHistory]] = []
+    source: Optional[str] = None
+
+
+class ConversationType(str, Enum):
+    """
+    Conversation types for holmes_conversation action
+    """
+
+    ISSUE = "issue"
+
+
+class HolmesChatParams(HolmesParams):
+    """
+    :var ask: User's prompt for holmes
+    """
+
+    ask: str
+    conversation_history: Optional[list[dict]] = None
+    render_graph_images: bool = False
+
+
+class HolmesIssueChatParams(HolmesChatParams):
+    resource: Optional[ResourceInfo] = ResourceInfo()
+    context: HolmesIssueChatParamsContext
+
+
+class HolmesConversationParams(HolmesParams):
+    """
+    :var resource: The resource related to this investigation. A resource has a `name` and `kind`, and may have `namespace` and `node`
+    :var ask: Override question to ask holmes
+    :var context: Additional information that can assist with the investigation
+    :var conversation_type: Type of a conversation issue/service/generic_ask (ConversationType)
+    """
+
+    ask: str
+    resource: Optional[ResourceInfo] = ResourceInfo()
+    context: HolmesOldConversationIssueContext
+    conversation_type: ConversationType
+    include_tool_calls: bool = True
+    include_tool_call_results: bool = True
+
+
+class HolmesWorkloadHealthParams(HolmesParams):
+    """
+    :var ask: Override question to ask holmes
+    :var resource: The resource related to this investigation. A resource has a `name` and `kind`, and may have `namespace` and `node`
+    :var alert_history: fetch historical alert data on the resource
+    :var alert_history_since_hours: Timespan of historic data to use in hours. 24 by default.
+    :var stored_instrucitons: Use remote instructions specified for the workload.
+    :var instructions: List of extra instructions to supply.
+    :var silent_healthy: Does not create findings in the case of healthy workload.
+
+    :example ask: What are all the issues in my cluster right now?
+    """
+
+    ask: Optional[str]
+    resource: Optional[ResourceInfo] = ResourceInfo()
+    alert_history: bool = True
+    alert_history_since_hours: float = 24
+    stored_instrucitons: bool = True
+    instructions: List[str] = []
+    include_tool_calls: bool = True
+    include_tool_call_results: bool = True
+    silent_healthy: bool = False
+
+
+class HolmesWorkloadHealthChatParams(HolmesParams):
+    """
+    :var ask: User's prompt for holmes
+    :var workload_health_result: Result from the workload health check
+    :var resource: The resource related to the initial investigation
+    :var conversation_history: List of previous user prompts and responses.
+    """
+
+    ask: str
+    workload_health_result: HolmesInvestigationResult
+    resource: ResourceInfo
+    conversation_history: Optional[list[dict]] = None
+
+
+
+class NamespacedResourcesParams(ActionParams):
+    """
+    :var name: Resource name
+    :var namespace: Resource namespace
+    """
+
+    name: str
+    namespace: str
+
+
 class PodRunningParams(ActionParams):
     """
     :var custom_annotations: custom annotations to be used for the running pod/job
@@ -118,6 +310,7 @@ class PrometheusParams(ActionParams):
     :var prometheus_url: Prometheus url. If omitted, we will try to find a prometheus instance in the same cluster
     :var prometheus_auth: Prometheus auth header to be used in Authorization header. If omitted, we will not add any auth header
     :var prometheus_url_query_string: Additional query string parameters to be appended to the Prometheus connection URL
+    :var prometheus_additional_headers: additional HTTP headers (if defined) to add to every prometheus query
     :var prometheus_additional_labels: A dictionary of additional labels needed for multi-cluster prometheus
     :var add_additional_labels: adds the additional labels (if defined) to the query
 
@@ -133,6 +326,7 @@ class PrometheusParams(ActionParams):
     prometheus_url: Optional[str] = None
     prometheus_auth: Optional[SecretStr] = None
     prometheus_url_query_string: Optional[str] = None
+    prometheus_additional_headers: Optional[Dict[str, str]] = None
     prometheus_additional_labels: Optional[Dict[str, str]] = None
     add_additional_labels: bool = True
     prometheus_graphs_overrides: Optional[List[OverrideGraph]] = None
@@ -173,6 +367,14 @@ class PrometheusDuration(BaseModel):
     duration_minutes: int
 
 
+class GraphEnricherParams(PrometheusParams):
+    """
+    :var graph_duration_minutes: the duration of the query in minutes
+    """
+
+    graph_duration_minutes: int = 60
+
+
 class PrometheusDateRange(BaseModel):
     """
     :var starts_at: the start date/time for the query
@@ -335,12 +537,23 @@ class OOMGraphEnricherParams(ResourceGraphEnricherParams):
     delay_graph_s: int = 0
 
 
-class OomKillParams(OOMGraphEnricherParams):
+class OomKillParams(OOMGraphEnricherParams, PodRunningParams):
     attach_logs: Optional[bool] = False
     container_memory_graph: Optional[bool] = False
     node_memory_graph: Optional[bool] = False
+    dmesg_log: Optional[bool] = False
 
-    def __init__(self, attach_logs: Optional[bool] = False, container_memory_graph: Optional[bool] = False,
-                 node_memory_graph: Optional[bool] = False, **kwargs):
-        super().__init__(attach_logs=attach_logs, container_memory_graph=container_memory_graph,
-                         node_memory_graph=node_memory_graph, resource_type=ResourceChartResourceType.Memory.name, **kwargs)
+    def __init__(
+        self,
+        attach_logs: Optional[bool] = False,
+        container_memory_graph: Optional[bool] = False,
+        node_memory_graph: Optional[bool] = False,
+        **kwargs,
+    ):
+        super().__init__(
+            attach_logs=attach_logs,
+            container_memory_graph=container_memory_graph,
+            node_memory_graph=node_memory_graph,
+            resource_type=ResourceChartResourceType.Memory.name,
+            **kwargs,
+        )
