@@ -7,37 +7,49 @@ import time
 from datetime import datetime
 from typing import Any, Dict, List, Optional, Union
 
+import requests
 from hikaru.model.rel_1_26 import DaemonSet, Deployment, Job, Node, Pod, ReplicaSet, StatefulSet
-from kubernetes.client import V1Node, V1NodeCondition, V1NodeList, V1Taint
-
-from robusta.core.discovery.discovery import DISCOVERY_STACKTRACE_TIMEOUT_S, Discovery, DiscoveryResults
+from robusta.core.model.namespaces import NamespaceMetadata, ResourceCount
+from robusta.core.discovery.discovery import DISCOVERY_STACKTRACE_TIMEOUT_S, Discovery, DiscoveryResults, ResourceAccessForbiddenError
 from robusta.core.discovery.top_service_resolver import TopLevelResource, TopServiceResolver
+from robusta.core.discovery.utils import from_api_server_node
+from robusta.core.model.base_params import HolmesParams
 from robusta.core.model.cluster_status import ActivityStats, ClusterStats, ClusterStatus
 from robusta.core.model.env_vars import (
     CLUSTER_STATUS_PERIOD_SEC,
     DISCOVERY_CHECK_THRESHOLD_SEC,
     DISCOVERY_PERIOD_SEC,
     DISCOVERY_WATCHDOG_CHECK_SEC,
+    HOLMES_ENABLED,
     MANAGED_CONFIGURATION_ENABLED,
 )
 from robusta.core.model.helm_release import HelmRelease
 from robusta.core.model.jobs import JobInfo
 from robusta.core.model.k8s_operation_type import K8sOperationType
 from robusta.core.model.namespaces import NamespaceInfo
-from robusta.core.model.nodes import NodeInfo, NodeSystemInfo
-from robusta.core.model.pods import PodResources
+from robusta.core.model.nodes import NodeInfo
+from robusta.core.model.openshift_group import OpenshiftGroup
 from robusta.core.model.services import ServiceInfo
 from robusta.core.pubsub.event_subscriber import EventHandler
 from robusta.core.reporting.base import Finding
 from robusta.core.reporting.consts import ScanState, ScanType
 from robusta.core.sinks.robusta.discovery_metrics import DiscoveryMetrics
-from robusta.core.sinks.robusta.prometheus_health_checker import PrometheusHealthChecker
+from robusta.core.sinks.robusta.prometheus_discovery_utils import PrometheusDiscoveryUtils
 from robusta.core.sinks.robusta.robusta_sink_params import RobustaSinkConfigWrapper, RobustaToken
 from robusta.core.sinks.robusta.rrm.rrm import RRM
 from robusta.core.sinks.sink_base import SinkBase
+from robusta.integrations.prometheus.utils import HolmesDiscovery
 from robusta.integrations.receiver import ActionRequestReceiver
 from robusta.runner.web_api import WebApi
 from robusta.utils.stack_tracer import StackTracer
+from robusta.core.model.env_vars import ROBUSTA_API_ENDPOINT
+from cachetools import TTLCache
+
+RUNNER_GET_HOLMES_SLACKBOT_INFO = f"{ROBUSTA_API_ENDPOINT}/api/holmes/integrations/slack/runner"
+HOLMES_SLACKBOT_CACHE_TTL = int(os.getenv("HOLMES_SLACKBOT_CACHE_TTL", 15 * 60))
+
+# Define the cache with a single slot and the configured TTL
+_holmes_slackbot_cache = TTLCache(maxsize=1, ttl=HOLMES_SLACKBOT_CACHE_TTL)
 
 
 class RobustaSink(SinkBase, EventHandler):
@@ -50,6 +62,8 @@ class RobustaSink(SinkBase, EventHandler):
         self.token = sink_config.robusta_sink.token
         self.ttl_hours = sink_config.robusta_sink.ttl_hours
         self.persist_events = sink_config.robusta_sink.persist_events
+        self.namespace_monitored_resources = sink_config.robusta_sink.namespaceMonitoredResources
+        self.namespace_discovery_seconds = sink_config.robusta_sink.namespace_discovery_seconds
 
         robusta_token = RobustaToken(**json.loads(base64.b64decode(self.token)))
         if self.account_id != robusta_token.account_id:
@@ -76,20 +90,21 @@ class RobustaSink(SinkBase, EventHandler):
         self.first_prometheus_alert_time = 0
         self.last_send_time = 0
         self.__discovery_period_sec = DISCOVERY_PERIOD_SEC
-
-        global_config = self.get_global_config()
-        self.__prometheus_health_checker = PrometheusHealthChecker(
-            discovery_period_sec=self.__discovery_period_sec, global_config=global_config
+        self.prometheus_real_status_reported = False
+        self.__errors = []
+        self.__prometheus_discovery_util = PrometheusDiscoveryUtils(
+            discovery_period_sec=self.__discovery_period_sec, registry=registry
         )
         self.__rrm_checker = RRM(dal=self.dal, cluster=self.cluster_name, account_id=self.account_id)
         self.__pods_running_count: int = 0
-        self.__update_cluster_status()  # send runner version initially, then force prometheus alert time periodically.
 
         self.registry.subscribe("scan_updated", self)
+        self.registry.subscribe("config_reload", self)
 
         # start cluster discovery
         self.__active = True
         self.__services_cache: Dict[str, ServiceInfo] = {}
+        self.__openshift_groups_cache: Dict[str, OpenshiftGroup] = {}
         self.__nodes_cache: Dict[str, NodeInfo] = {}
         self.__namespaces_cache: Dict[str, NamespaceInfo] = {}
         self.__jobs_cache: Optional[Dict[str, JobInfo]] = {}
@@ -97,14 +112,14 @@ class RobustaSink(SinkBase, EventHandler):
         self.__jobs_cache_initialized: bool = False
         self.__helm_releases_cache: Optional[Dict[str, HelmRelease]] = None
         self.__init_service_resolver()
-        self.__thread = threading.Thread(target=self.__discover_cluster)
         self.__watchdog_thread = threading.Thread(target=self.__discovery_watchdog)
-        self.__thread.start()
         self.__watchdog_thread.start()
 
     def handle_event(self, event_name: str, **kwargs):
         if event_name == "scan_updated":
             self._on_scan_updated(**kwargs)
+        elif event_name == "config_reload":
+            self._on_config_reload(**kwargs)
         else:
             logging.warning("RobustaSink subscriber called with unknown event")
 
@@ -116,6 +131,14 @@ class RobustaSink(SinkBase, EventHandler):
 
         self.dal.set_scan_state(scan_id, state, metadata)
 
+    def _on_config_reload(self) -> None:
+        # make sure cluster status and periodic check start after all config has been reloaded successfuly.
+        self.__update_cluster_status()  # send runner version initially, then force prometheus alert time periodically.
+
+        if not hasattr(self, '_thread'):
+            self._thread = threading.Thread(target=self.__discover_cluster, daemon=True)
+            self._thread.start()
+        
     def set_cluster_active(self, active: bool):
         self.dal.set_cluster_active(active)
 
@@ -151,6 +174,12 @@ class RobustaSink(SinkBase, EventHandler):
             for service in self.dal.get_active_services():
                 self.__services_cache[service.get_service_key()] = service
 
+    def __assert_openshift_groups_cache_initialized(self):
+        if not self.__openshift_groups_cache:
+            logging.info("Initializing openshift groups cache")
+            for os_group in self.dal.get_active_openshift_groups():
+                self.__openshift_groups_cache[os_group.get_service_key()] = os_group
+
     def __assert_node_cache_initialized(self):
         if not self.__nodes_cache:
             logging.info("Initializing nodes cache")
@@ -299,6 +328,63 @@ class RobustaSink(SinkBase, EventHandler):
         except Exception:
             logging.error("Error occurred while sending `helm release` trigger event", exc_info=True)
 
+    def __discover_custom_namespaced_resources(self, namespaces: List[NamespaceInfo]):
+        if not self.namespace_monitored_resources:
+            return
+
+        try:
+            # Step 1: Collect counts in a temporary map
+            resource_map = {}  # type: Dict[str, List[ResourceCount]]
+
+            for resource in self.namespace_monitored_resources:
+                try:
+                    results = Discovery.count_resources(
+                        kind=resource.kind,
+                        api_group=resource.apiGroup,
+                        version=resource.apiVersion
+                    )
+
+                    for namespace_name, count in results.items():
+                        if namespace_name not in resource_map:
+                            resource_map[namespace_name] = []
+
+                        resource_map[namespace_name].append(ResourceCount(
+                            kind=resource.kind,
+                            apiVersion=resource.apiVersion,
+                            apiGroup=resource.apiGroup,
+                            count=count
+                        ))
+
+                except ResourceAccessForbiddenError as e:
+                    logging.warning(f"Skipping resource {resource.kind} due to insufficient permissions: {e}")
+                except Exception as e:
+                    logging.exception(f"Unexpected error counting resource {resource.kind}: {e}")
+
+            # Step 2: Apply metadata to matching NamespaceInfo entries
+            for ns in namespaces:
+                if ns.name in resource_map:
+                    if not ns.metadata:
+                        ns.metadata = NamespaceMetadata(resources=[])
+                    ns.metadata.resources.extend(resource_map[ns.name])
+                    
+            logging.info("Discovered Namespaced custom resources")
+            return namespaces
+
+        except Exception as e:
+            logging.exception(f"Namespace discovery failed: {e}")
+
+    def __add_cached_namespace_metadata(self, namespaces: List[NamespaceInfo]):
+        discovered_namespaces = {namespace.name: namespace for namespace in namespaces}
+        updated_namespaces: List[NamespaceInfo] = []
+
+        for namespace_name, namespace in discovered_namespaces.items():
+            cached_namespace = self.__namespaces_cache.get(namespace_name)
+            if cached_namespace:
+                namespace.metadata = cached_namespace.metadata
+            updated_namespaces.append(namespace)
+        
+        return updated_namespaces
+
     def __discover_resources(self) -> DiscoveryResults:
         # discovery is using the k8s python API and not Hikaru, since it's performance is 10 times better
         try:
@@ -308,7 +394,7 @@ class RobustaSink(SinkBase, EventHandler):
             self.__publish_new_services(results.services)
             if results.nodes:
                 self.__assert_node_cache_initialized()
-                self.__publish_new_nodes(results.nodes, results.node_requests)
+                self.__publish_new_nodes(results.nodes)
 
             self.__assert_jobs_cache_initialized()
             self.__publish_new_jobs(results.jobs)
@@ -317,9 +403,20 @@ class RobustaSink(SinkBase, EventHandler):
             self.__publish_new_helm_releases(results.helm_releases)
 
             self.__assert_namespaces_cache_initialized()
-            self.__publish_new_namespaces(results.namespaces)
+            namespaces = results.namespaces
+            if self.namespace_monitored_resources and (time.time() - self.last_namespace_discovery) >= self.namespace_discovery_seconds:
+                namespaces = self.__discover_custom_namespaced_resources(namespaces)
+                self.last_namespace_discovery = time.time()
+            elif self.namespace_monitored_resources:
+                namespaces = self.__add_cached_namespace_metadata(namespaces)
+            self.__publish_new_namespaces(namespaces)
 
             self.__pods_running_count = results.pods_running_count
+
+            if results.openshift_groups:
+                self.__assert_openshift_groups_cache_initialized()
+                self.__publish_new_openshift_groups(results.openshift_groups)
+
             # save the cached services for the resolver.
             RobustaSink.__save_resolver_resources(
                 list(self.__services_cache.values()), list(self.__jobs_cache.values())
@@ -334,77 +431,15 @@ class RobustaSink(SinkBase, EventHandler):
                 f"Failed to run publish discovery for {self.sink_name}",
                 exc_info=True,
             )
+        finally:
+            if Discovery.out_of_memory_detected and "ERROR_DISCOVERY_OOM" not in self.__errors:
+                self.__errors.append("ERROR_DISCOVERY_OOM")
 
-    @classmethod
-    def __to_taint_str(cls, taint: V1Taint) -> str:
-        return f"{taint.key}={taint.value}:{taint.effect}"
-
-    @classmethod
-    def __to_active_conditions_str(cls, conditions: List[V1NodeCondition]) -> str:
-        if not conditions:
-            return ""
-        return ",".join(
-            [
-                f"{condition.type}:{condition.status}"
-                for condition in conditions
-                if condition.status != "False" or condition.type == "Ready"
-            ]
-        )
-
-    @classmethod
-    def __to_node_info(cls, node: Union[V1Node, Node]) -> Dict:
-        info = getattr(node.status, "node_info", None) or getattr(node.status, "nodeInfo", None)
-        node_info = {}
-        node_info["system"] = NodeSystemInfo(**info.to_dict()).dict() if info else {}
-        node_info["labels"] = node.metadata.labels or {}
-        node_info["annotations"] = node.metadata.annotations or {}
-        node_info["addresses"] = [addr.address for addr in node.status.addresses] if node.status.addresses else []
-        return node_info
-
-    @classmethod
-    def __from_api_server_node(
-        cls, api_server_node: Union[V1Node, Node], pod_requests_list: List[PodResources]
-    ) -> NodeInfo:
-        addresses = api_server_node.status.addresses or []
-        external_addresses = [address for address in addresses if "externalip" in address.type.lower()]
-        external_ip = ",".join([addr.address for addr in external_addresses])
-        internal_addresses = [address for address in addresses if "internalip" in address.type.lower()]
-        internal_ip = ",".join([addr.address for addr in internal_addresses])
-        node_taints = api_server_node.spec.taints or []
-        taints = ",".join([cls.__to_taint_str(taint) for taint in node_taints])
-        capacity = api_server_node.status.capacity or {}
-        allocatable = api_server_node.status.allocatable or {}
-        # V1Node and Node use snake case and camelCase respectively, handle this for more than 1 word attributes.
-        creation_ts = getattr(api_server_node.metadata, "creation_timestamp", None) or getattr(
-            api_server_node.metadata, "creationTimestamp", None
-        )
-        version = getattr(api_server_node.metadata, "resource_version", None) or getattr(
-            api_server_node.metadata, "resourceVersion", None
-        )
-        return NodeInfo(
-            name=api_server_node.metadata.name,
-            node_creation_time=str(creation_ts),
-            internal_ip=internal_ip,
-            external_ip=external_ip,
-            taints=taints,
-            conditions=cls.__to_active_conditions_str(api_server_node.status.conditions),
-            memory_capacity=PodResources.parse_mem(capacity.get("memory", "0Mi")),
-            memory_allocatable=PodResources.parse_mem(allocatable.get("memory", "0Mi")),
-            memory_allocated=sum([req.memory for req in pod_requests_list]),
-            cpu_capacity=PodResources.parse_cpu(capacity.get("cpu", "0")),
-            cpu_allocatable=PodResources.parse_cpu(allocatable.get("cpu", "0")),
-            cpu_allocated=round(sum([req.cpu for req in pod_requests_list]), 3),
-            pods_count=len(pod_requests_list),
-            pods=",".join([pod_req.pod_name for pod_req in pod_requests_list]),
-            node_info=cls.__to_node_info(api_server_node),
-            resource_version=int(version) if version else 0,
-        )
-
-    def __publish_new_nodes(self, current_nodes: V1NodeList, node_requests: Dict[str, List[PodResources]]):
+    def __publish_new_nodes(self, current_nodes: List[NodeInfo]):
         # convert to map
         curr_nodes = {}
-        for node in current_nodes.items:
-            curr_nodes[node.metadata.name] = node
+        for node in current_nodes:
+            curr_nodes[node.name] = node
 
         # handle deleted nodes
         updated_nodes: List[NodeInfo] = []
@@ -415,8 +450,7 @@ class RobustaSink(SinkBase, EventHandler):
 
         # new or changed nodes
         for node_name in curr_nodes.keys():
-            pod_requests = node_requests.get(node_name, [])  # if all the pods on the node have no requests
-            updated_node = self.__from_api_server_node(curr_nodes.get(node_name), pod_requests)
+            updated_node = curr_nodes.get(node_name)
             if self.__nodes_cache.get(node_name) != updated_node:  # node not in the cache, or changed
                 updated_nodes.append(updated_node)
                 self.__nodes_cache[node_name] = updated_node
@@ -439,6 +473,13 @@ class RobustaSink(SinkBase, EventHandler):
         if service_key:
             self.dal.remove_deleted_service(service_key)
 
+    def __safe_delete_openshift_group(self, service_key):
+        self.__services_cache.pop(service_key, None)
+
+        # could be case where it is not in cache but is in db, i.e. after cache reset
+        if service_key:
+            self.dal.remove_deleted_service(service_key)
+
     def __safe_delete_job(self, job_key):
         job_info = self.__jobs_cache.pop(job_key, None)
         if job_info:
@@ -492,15 +533,36 @@ class RobustaSink(SinkBase, EventHandler):
 
         self.dal.publish_helm_releases(helm_releases)
 
+    def get_holmes_model(self) -> Optional[str]:
+        global_config = self.get_global_config()
+        params: HolmesParams = HolmesParams(**global_config)
+        holmes_url = HolmesDiscovery().find_holmes_url(params.holmes_url)
+        if not holmes_url:
+            return None
+
+        try:
+            res = requests.get(f"{holmes_url}{HolmesDiscovery.MODEL_NAME_URL}")
+            res.raise_for_status()
+            model_name = res.json()["model_name"]
+            return model_name
+        except Exception:
+            return None
+
     def __update_cluster_status(self):
         self.last_send_time = time.time()
-        prometheus_health_checker_status = self.__prometheus_health_checker.get_status()
+        prometheus_health_checker_status = self.__prometheus_discovery_util.get_status()
+        holmes_model = self.get_holmes_model()
+
         activity_stats = ActivityStats(
             relayConnection=False,
             alertManagerConnection=prometheus_health_checker_status.alertmanager,
             prometheusConnection=prometheus_health_checker_status.prometheus,
             prometheusRetentionTime=prometheus_health_checker_status.prometheus_retention_time,
             managedPrometheusAlerts=MANAGED_CONFIGURATION_ENABLED,
+            holmesEnabled=HOLMES_ENABLED,
+            holmesModel=holmes_model,
+            clusterTimeZone=str(datetime.now().astimezone().tzinfo),
+            errors=self.__errors,
         )
 
         # checking the status of relay connection
@@ -524,7 +586,9 @@ class RobustaSink(SinkBase, EventHandler):
             )
 
             self.dal.publish_cluster_status(cluster_status)
-            self.dal.publish_cluster_nodes(cluster_stats.nodes, self.__pods_running_count)
+            avg_cpu = self.__prometheus_discovery_util.get_cluster_avg_cpu()
+            avg_mem = self.__prometheus_discovery_util.get_cluster_avg_memory()
+            self.dal.publish_cluster_nodes(cluster_stats.nodes, self.__pods_running_count, avg_cpu, avg_mem)
         except Exception:
             logging.exception(
                 f"Failed to run periodic update cluster status for {self.sink_name}",
@@ -562,33 +626,45 @@ class RobustaSink(SinkBase, EventHandler):
     def __discover_cluster(self):
         logging.info("Cluster discovery initialized")
         get_history = self.__should_run_history()
+        self.last_namespace_discovery = 0
         while self.__active:
             start_t = time.time()
             self.__periodic_cluster_status()
             discovery_results = self.__discover_resources()
+
             if get_history:
                 self.__get_events_history()
                 get_history = False
-
             if discovery_results and discovery_results.helm_releases:
                 self.__send_helm_release_events(release_data=discovery_results.helm_releases)
-
             duration = round(time.time() - start_t)
-            # for small cluster duration is discovery_period_sec. For bigger clusters, up to 5 min
             sleep_dur = min(max(self.__discovery_period_sec, 3 * duration), 300)
+
             logging.debug(f"Discovery duration: {duration} next discovery in {sleep_dur}")
             time.sleep(sleep_dur)
 
         logging.info(f"Service discovery for sink {self.sink_name} ended.")
 
+
     def __periodic_cluster_status(self):
         first_alert = False
+
+        # Check if it's the first time an alert occurred
         if self.registry.get_telemetry().last_alert_at and self.first_prometheus_alert_time == 0:
             first_alert = True
             self.first_prometheus_alert_time = time.time()
-
-        if time.time() - self.last_send_time > CLUSTER_STATUS_PERIOD_SEC or first_alert:
+        finished_first_prometheus_discovery = (
+            self.__prometheus_discovery_util.first_checks_finished and not self.prometheus_real_status_reported
+        )
+        # Regular update logic: only when CLUSTER_STATUS_PERIOD_SEC has passed or on first alert
+        if (
+            (time.time() - self.last_send_time > CLUSTER_STATUS_PERIOD_SEC)
+            or first_alert
+            or finished_first_prometheus_discovery
+        ):
+            self.prometheus_real_status_reported = self.__prometheus_discovery_util.first_checks_finished
             self.__update_cluster_status()
+            self.last_send_time = time.time()
 
     def __publish_new_namespaces(self, namespaces: List[NamespaceInfo]):
         # convert to map
@@ -611,12 +687,41 @@ class RobustaSink(SinkBase, EventHandler):
 
         self.dal.publish_namespaces(updated_namespaces)
 
+    def __publish_new_openshift_groups(self, os_groups: List[OpenshiftGroup]):
+        # convert to map
+        curr_os_groups = {}
+        for os_group in os_groups:
+            curr_os_groups[os_group.get_service_key()] = os_group
+
+        # handle deleted openshift groups.
+        cache_keys = list(self.__openshift_groups_cache.keys())
+        updated_os_groups: List[OpenshiftGroup] = []
+        for os_group_key in cache_keys:
+            if not curr_os_groups.get(os_group_key):  # service doesn't exist any more, delete it
+                self.__safe_delete_openshift_group(os_group_key)
+
+        # new or changed services
+        for key in curr_os_groups.keys():
+            current_os_group = curr_os_groups[key]
+            cached_os_group = self.__openshift_groups_cache.get(key)
+
+            # prevent service updates if the resource version in the cache is lower than the new service
+            if cached_os_group and cached_os_group.resource_version > current_os_group.resource_version:
+                continue
+
+            # group not in the cache, or changed
+            if self.__openshift_groups_cache.get(key) != current_os_group:
+                updated_os_groups.append(current_os_group)
+                self.__openshift_groups_cache[key] = current_os_group
+
+        self.dal.persist_openshift_groups(updated_os_groups)
+
     def get_global_config(self) -> dict:
         return self.registry.get_global_config()
 
     def __update_node(self, new_node: Node, operation: K8sOperationType):
         with self.services_publish_lock:
-            new_info = self.__from_api_server_node(new_node, [])
+            new_info = from_api_server_node(new_node, [])
             if operation == K8sOperationType.CREATE:
                 name = new_node.metadata.name
                 self.__nodes_cache[name] = new_info
@@ -673,3 +778,26 @@ class RobustaSink(SinkBase, EventHandler):
                 self.__safe_delete_job(job_key)
                 self.__discovery_metrics.on_jobs_updated(1)
                 return
+
+    def is_holmes_slackbot_connected(self) -> bool:
+        if 'status' in _holmes_slackbot_cache:
+            return _holmes_slackbot_cache['status']
+        session_token = self.dal.get_session_token()
+        try:
+            message_json = {
+                "session_token": session_token,
+                "account_id": self.account_id,
+            }
+            response = requests.post(
+                RUNNER_GET_HOLMES_SLACKBOT_INFO,
+                json=message_json,
+                headers={"Content-Type": "application/json"}
+            )
+            response.raise_for_status()
+            is_connected = bool(response.json().get("integrations"))
+            _holmes_slackbot_cache['status'] = is_connected
+            return is_connected
+        except Exception as e:
+            logging.warning(f"Failed to get holmes slackbot info {e}")
+            _holmes_slackbot_cache['status'] = False
+            return False
\ No newline at end of file
