@@ -1,7 +1,8 @@
 import logging
-from typing import Optional
+from typing import Optional, Dict
 
 from requests.auth import HTTPBasicAuth
+from requests.exceptions import HTTPError
 from requests_toolbelt import MultipartEncoder
 
 from robusta.core.reporting.base import FindingStatus
@@ -34,6 +35,13 @@ class JiraClient:
         else:
             self.default_issue_type_id = self._get_default_issue_type()
 
+        if jira_params.epic:
+            self.epic = jira_params.epic
+        
+        if jira_params.assignee:
+            self.assignee = jira_params.assignee
+
+
         if jira_params.project_type_id_override:
             self.default_project_id = jira_params.project_type_id_override
         else:
@@ -51,6 +59,24 @@ class JiraClient:
                 f"Jira initialized successfully. Project: {self.default_project_id} issue type: {self.default_issue_type_id}"
             )
 
+        if jira_params.priority_mapping:
+            if logging.getLogger().getEffectiveLevel() <= logging.DEBUG:
+                self._validate_priorities(jira_params.priority_mapping)
+
+    def _validate_priorities(self, priority_mapping: Dict[str, str]) -> None:
+        """Validate that configured priorities exist in Jira"""
+        endpoint = "priority"
+        url = self._get_full_jira_url(endpoint)
+        available_priorities = self._call_jira_api(url) or []
+        available_priority_names = {p.get("name") for p in available_priorities}
+
+        for severity, priority in priority_mapping.items():
+            if priority not in available_priority_names:
+                logging.warning(
+                    f"Configured priority '{priority}' for severity '{severity}' "
+                    f"is not available in Jira. Available priorities: {available_priority_names}"
+                )
+
     def _get_full_jira_url(self, endpoint: str) -> str:
         return "/".join([self.params.url, _API_PREFIX, endpoint])
 
@@ -124,7 +150,7 @@ class JiraClient:
 
         issue = issues.get("issues")[0]
         if issues.get("total", 0) > 1:
-            logging.warn(f"More than one issue found for query: '{query}', picking most recent one")
+            logging.warning(f"More than one issue found for query: '{query}', picking most recent one")
             logging.debug(f"Picked issue '{issue}'")
 
         return issue
@@ -153,6 +179,23 @@ class JiraClient:
             return default_issue["id"]
         return None
 
+    def _resolve_priority(self, priority_name: str) -> dict:
+        """Resolve Jira priority:
+        1. User configured priority mapping (if defined)
+        2. Fallback to current behavior (use priority name as-is)
+
+        Returns:
+            dict: Priority field in format {"name": str}
+        """
+        # 1. Try user configured priority mapping
+        if hasattr(self, "params") and self.params.priority_mapping:
+            for severity, mapped_name in self.params.priority_mapping.items():
+                if mapped_name == priority_name:
+                    return {"name": mapped_name}
+
+        # 2. Fallback to current behavior
+        return {"name": priority_name}
+
     def list_issues(self, search_params: Optional[str] = None):
         endpoint = "search"
         search_params = search_params or ""
@@ -201,6 +244,13 @@ class JiraClient:
     def create_issue(self, issue_data, issue_attachments=None):
         endpoint = "issue"
         url = self._get_full_jira_url(endpoint)
+
+        # Add priority resolution if it exists
+        if "priority" in issue_data:
+            priority_name = issue_data["priority"].get("name")
+            if priority_name:
+                issue_data["priority"] = self._resolve_priority(priority_name)
+
         payload = {
             "update": {},
             "fields": {
@@ -223,9 +273,20 @@ class JiraClient:
         url = self._get_full_jira_url(endpoint)
 
         payload = {
-            "fields": {"summary": summary, "description": description},
+            "fields": {
+                "summary": summary,
+                "description": description,
+            }
         }
 
+        # Only add assignee if defined
+        if hasattr(self, 'assignee') and self.assignee:
+            payload["fields"]["assignee"] = {"id": self.assignee}
+
+        # Only add parent if epic is defined
+        if hasattr(self, 'epic') and self.epic:
+            payload["fields"]["parent"] = {"key": self.epic}
+
         logging.debug(f"Update issue '{issue_id}' with payload: {payload}")
         response = self._call_jira_api(url, http_method=HttpMethod.PUT, json=payload) or {}
         logging.debug(response)
